/**
 * @file
 * VuoList_VuoVerticalReflection C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoVerticalReflection_H
#define VuoList_VuoVerticalReflection_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoVerticalReflection VuoList_VuoVerticalReflection
 * A list of @ref VuoVerticalReflection elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoVerticalReflection_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoVerticalReflection elements.
 */
typedef const struct VuoList_VuoVerticalReflection_struct { void *l; } * VuoList_VuoVerticalReflection;
/// @}
#endif

/**
 * Creates a new list of @ref VuoVerticalReflection elements.
 */
VuoList_VuoVerticalReflection VuoListCreate_VuoVerticalReflection(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoVerticalReflection VuoListCopy_VuoVerticalReflection(const VuoList_VuoVerticalReflection list);

/**
 * Returns the @ref VuoVerticalReflection at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoVerticalReflection VuoListGetValue_VuoVerticalReflection(const VuoList_VuoVerticalReflection list, const unsigned long index);

/**
 * Changes the @ref VuoVerticalReflection at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoVerticalReflection(const VuoList_VuoVerticalReflection list, const VuoVerticalReflection value, const unsigned long index);

/**
 * Inserts the `VuoVerticalReflection` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoVerticalReflection(const VuoList_VuoVerticalReflection list, const VuoVerticalReflection value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoVerticalReflection(VuoList_VuoVerticalReflection list, const VuoVerticalReflection value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoVerticalReflection(VuoList_VuoVerticalReflection list, const VuoVerticalReflection value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoVerticalReflection(VuoList_VuoVerticalReflection list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoVerticalReflection(VuoList_VuoVerticalReflection list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoVerticalReflection(VuoList_VuoVerticalReflection list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoVerticalReflection(VuoList_VuoVerticalReflection list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoVerticalReflection(VuoList_VuoVerticalReflection list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoVerticalReflection(VuoList_VuoVerticalReflection list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoVerticalReflection(VuoList_VuoVerticalReflection list);

/**
 * Removes the `VuoVerticalReflection` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoVerticalReflection(VuoList_VuoVerticalReflection list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoVerticalReflection(const VuoList_VuoVerticalReflection list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoVerticalReflection VuoList_VuoVerticalReflection_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoVerticalReflection_getJson(const VuoList_VuoVerticalReflection value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoVerticalReflection_getSummary(const VuoList_VuoVerticalReflection value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoVerticalReflection VuoList_VuoVerticalReflection_makeFromString(const char *str);
char * VuoList_VuoVerticalReflection_getString(const VuoList_VuoVerticalReflection value);
/// @}

/**
 * @}
 */

#endif
